#!/env/python3
'''
Generates attachement for TST 010 based on index files in the api-tests repository

  Usage: python3 create_sols.py <path-to-api-tests>   <commit-or-tag>
'''

import os
import sys
import csv

from robot.api import TestSuiteBuilder

import testspec
import config
import lib

indexes = {
    "SOL002" : 'sol_002_index.csv',
    "SOL003" : 'sol_003_index.csv',
    "SOL005" : 'sol_005_index.csv',
}

templates = {
    "SOL002" : 'Tests-SOL002-VeVnfm_template.docx',
    "SOL003" : 'Tests-SOL003-OrVnfm_template.docx',
    "SOL005" : 'Tests-SOL005-OsMaNfvo_template.docx'
}

outnames = {
    "SOL002" : 'Tests-SOL002-VeVnfm.docx',
    "SOL003" : 'Tests-SOL003-OrVnfm.docx',
    "SOL005" : 'Tests-SOL005-OsMaNfvo.docx',
}

def is_interface_heading(ref):
    '''Check if a reference has heading level 3'''
    return ref.lvl() == 3


def is_endpoint_heading(ref):
    '''Predicate to check if a reference has heading level 4'''
    return ref.lvl() == 4

def generate_new_clause(row, ref, spec, all_tests_root, commit_id, solspec, current_folder):
    '''
    Generates a new clause, either Void or for a Test Suite (i.e. a .robot file)
    '''
    if  "Void" in row["fln"]:
        lib.generate_void_clause(ref, row["title"], spec, solspec)
    else:
        path = os.path.join(all_tests_root, solspec, current_folder, row["fln"])
        suite = TestSuiteBuilder().build(path)
        lib.gen_api_suite(ref, suite, spec, current_folder, commit_id, solspec)

def load_spec_index(root_dir, base_spec):
    '''
    Load the index from the file, parses the CSV and applies strip 
    '''

    spec_idx = None

    index_filename = open(os.path.join(root_dir, "indexes", indexes[base_spec]), 'r')
    spec_idx = csv.DictReader(
        index_filename,
        skipinitialspace=True, 
        fieldnames=["ref", "title", "fln"]
    )

    if spec_idx is None:
        print("ERROR OPENING INDEX: {}".format(indexes[base_spec]))
        return []

    spec_idx = \
        (dict((k, v.strip()) for k, v in row.items() if v) for row in spec_idx if len(row) >= 3)

    return spec_idx

def create_test_for_sol(basedoc, spec_idx, all_tests_root, commit_id, output_dir):
    '''
    Loads index file for a base specification and iterates to produce 
    all clauses, 
    '''
    if not basedoc in indexes:
        print("Error: Wrong SOL Specification: {}".format(basedoc))
        return

    spec = testspec.TestSpec(templates[basedoc])
    current_folder = None

    for item in spec_idx:
        try:
            ref = lib.DocReference.from_str(item["ref"])

            if is_interface_heading(ref):
                spec.add_heading_from_ref(item["title"], ref)
                if "Void" in item["fln"]:
                    spec.add_void_paragraph()
                else:
                    current_folder = item["fln"]

            elif is_endpoint_heading(ref):
                if item["fln"] is None or current_folder is None:
                    print(item)
                    raise Exception()
                generate_new_clause(
                    item, ref, spec, all_tests_root, commit_id, basedoc, current_folder
                )

        except Exception as err:
            print(item, all_tests_root, basedoc, current_folder)
            raise err

    outfn = os.path.join(output_dir,outnames[basedoc])
    print("Writing {} definitions to: {}".format(basedoc, outfn))
    spec.save(outfn)

if __name__ == "__main__":

    if len(sys.argv) < 2:
        print("Usage: robot2doc <sol_robot_root_dir> <commit-id>")
        sys.exit(-1)

    ROOT = sys.argv[1]
    COMMIT_ID = sys.argv[2] if len(sys.argv) > 2 else None

    print("Received arguments: ", sys.argv)

    print("Generating entire SOL tests")

    sol_specs = ["SOL002"] #, "SOL003", "SOL005"]

    for solspec in sol_specs:
        spec_index = load_spec_index(ROOT, solspec)
        create_test_for_sol(solspec, spec_index, ROOT, COMMIT_ID, "..")

